/* figpar.h
   --------
   Author: Devin Teske
   Date: December 31, 2002
   Version: 1.1

   Configuration Parsing (fig-par)  routines.  This utility
   provides a very flexible way to interpret  configuration
   files in plain text format; UTF-8 only.
*/

#ifndef HAVE_FIGPAR_H
#define HAVE_FIGPAR_H

#ifdef __cplusplus
extern "C" {
#endif

/* custom includes */
#include "config.h"      /* C preprocessor macros */ 

/* system includes */
#include <sys/types.h>   /* data types from the system */



/* macros */

/* legacy lseek whence macro support */
#ifndef SEEK_SET
#ifdef L_SET
   #define SEEK_SET L_SET
#else
   #define SEEK_SET 0
#endif
#endif

#ifndef SEEK_CUR
#ifdef L_INCR
   #define SEEK_CUR L_INCR
#else
   #define SEEK_CUR 1
#endif
#endif

#ifndef SEEK_END
#ifdef L_XTND
   #define SEEK_END L_XTND
#else
   #define SEEK_END 2
#endif
#endif



/* data types */

/* cfgvalue_t is a 4 byte union that allows you to store
   varied types of data in a single common container. */
typedef union {

   char *str;            /* a pointer to a null terminated string */
   int32_t num;          /* a signed 32-bit integer value */
   u_int32_t boolean:1;  /* boolean integer value (0 or 1) */
   u_int32_t u_num;      /* an unsigned 32-bit integer value */ 
   char **strarray;      /* pointer to an array of strings */

} cfgvalue_t;

/* anatomy of a config file option */
typedef struct {

   char *directive;    /* key word to locate in config file */
   int type;           /* type of value that the option is:
                          0: unused (mostly for action based directives)
                          1: boolean
                          2: signed 32 bit integer
                          3: unsigned 32 bit integer
                          4: string pointer
                          5: string array pointer */

   cfgvalue_t value;   /* default value (action should overwrite
                          this value replacing the default) */

   int (*action)(u_int32_t, char *, char *);
                       /* function pointer; action to be taken
                          when the directive is found */

} config_t;



/* functions */

/* get_config_option
   -----------------
   Searches for a string in an array and returns the struct
   that contains the string as the directive.

   This is to eliminate dependency on the index position of
   an  item in the array,  since the index position is more
   apt to be changed.

   Returns  a pointer to the actual structure.  If no match
   is found, a pointer to a dummy array is returned.
*/
config_t* get_config_option (config_t [], char *);

/* parse_config
   ------------
   takes 3 parameters. array of options to search for. path
   to the file to parse. pointer to the function to execute
   for unknown options. returns zero on success.
*/
int parse_config (config_t [], char *, int (*)(u_int32_t, char *, char *));



#ifdef __cplusplus
}
#endif

#endif /* HAVE_FIGPAR_H */
