/*

	Sockt.h - Throttled Socket Operations v1

	General Notes:
	Since throttling changes the amount of data being sent out at a
	time, care must be taken to how you add throttling capabilities
	to your application.

	Small packets being sent through *1* socket from *MORE THAN 1*
	source within your program cannot be fragmented, and you must use
	writet() and readt(). However, if you can avoid this situation,
	I suggest you do so.

	However, the recommended use of throttling is for file transfers,
	where you do not positively have to have it sent all at once, since
	there is only one source sending packets, since only one source is
	reading the file to transfer. 

	A source can be a single thread, or a single point in the program,
	multiple sources occur when you could be trying to send two packets
	across a socket from different parts of the program before one of
	the packets is done being sent.
 */

#include <stddef.h>

#define THROTTLE_THREADS

#define UNTHROTTLED		0xFFFFFFFF

#define BANK_MSEC		1000
#define BANK_SEC		1000000
#define MAX_BANK_MSECS  (100)
#define MAX_BANK_USECS  (BANK_MSEC * MAX_BANK_MSECS)

/*** Declarations ***/

/*
 *	setincomingmax() and setoutgoingmax()
 *
 *	Simply sets the maximums easily.
 */
void setoutgoingmax(unsigned long max);
void setincomingmax(unsigned long max);

/*
 *  sendt() and recvt()
 *
 *  Notes: Writes data, returning even if it cannot all be sent.
 *			If you MUST block until it is all sent, use writet/readt
 *			instead.
 */
ssize_t should_sendt();
ssize_t should_recvt();
ssize_t sendt(int socket, const void *data, size_t len, int flags);
ssize_t recvt(int socket, const void *data, size_t len, int flags);

/*
 *	writet() and readt()
 *
 *	Notes: Blocks until all requested data is written or read. It
 *			is not recommended that you use this unless sending small
 *			packets over a throttled socket which cannot become
 *			fragmented.
 */
ssize_t writet(int socket, const void *data, size_t len);
ssize_t readt(int socket, const void *data, size_t len);


