/* figpar.c
   --------
   figpar, the con-fig par-ser. heh

   This  file  contains code to parse complex configuration
   files.  It steps through the file, reading in values for
   directives declared in an array of pre-defined structur-
   es  outlining  the anatomy of configuration options.  It
   uses  function  pointers to allow extensibility.  Rather
   than  re-programming  the function that reads and parses
   the file itself, you can make functions that perform sp-
   ecific tasks per directive.  You can also perform a task
   for unknown directives not declared in your array.

   The way that this parser is written is very simple,  and
   how  complex  of  a set of features you want to allow in
   your file is dependent on the functions you have it exe-
   cute.

   It is possible to read apache style config files accura-
   tely, .htaccess files, and other styles.

   You can find the external declarations for these methods
   in (from the root of the lazari folder) `src/include/fi-
   gpar.h'.  To  use  these methods and functions,  include
   the  line  `#include "figpar.h"'  into  the  top of your
   source file.
*/


/* custom includes */
#include "config.h"           /* C preprocessor macros */ 
#include "main.h"          /* PATH_MAX and NAME_MAX */
#include "figpar.h"      /* local header */
#include "string_m.h"    /* string manipulation routines */
#ifndef HAVE_REALPATH
#include "realpath.h"         /* local declaration of realpath */
#endif

/* system includes */
#include <stdlib.h>           /* malloc and realloc */
#include <fcntl.h>            /* open */
#include <unistd.h>           /* read lseek and close */
#include <errno.h>            /* error detection */
#include <ctype.h>            /* isspace, tolower */
#include <fnmatch.h>          /* fnmatch */
#include <string.h>           /* strcmp */



/* variables */
config_t dummy_config = {0,0,{0},0};



/* functions */

/* get_config_option
   -----------------
   Searches for a string in an array and returns the struct
   that contains the string as the directive.

   This is to eliminate dependency on the index position of
   an  item in the array,  since the index position is more
   apt to be changed.

   Returns  a pointer to the actual structure.  If no match
   is found, a pointer to a dummy array is returned.
*/
config_t* get_config_option (config_t option[], char *directive)
{
   u_int32_t n;

   /* loop through the array, return the index of the first match */
   for (n = 0; option[n].directive; n++)
      if (!strcmp(option[n].directive, directive)) return &option[n];

   /* re-initialize the dummy variable in case it was written to */
   dummy_config.directive = 0;
   dummy_config.type = 0;
   dummy_config.value.u_num = 0;
   dummy_config.action = 0;

   return &dummy_config;
}



/* parse_config
   ------------
   takes 3 parameters. array of options to search for. path
   to the file to parse. pointer to the function to execute
   for unknown options. returns zero on success.
*/
int parse_config (config_t option[], char *path,
int (*unknown)(u_int32_t, char *, char *))
{
   char rpath[PATH_MAX], p[2], *t;
   char *directive, *value;
   int fd, err; ssize_t r = 1;
   u_int8_t comment = 0, quote, end, found;
   u_int32_t dsize, vsize, n, x, line = 1;
   off_t curpos, charpos;

   /* initialize strings */
   directive = value = 0;
   vsize = dsize = 0;

   /* resolve the file path */
   /* WARNING:  According  to  POSIX  compliancy  standards
      realpath() is not considered safe when used in a sig-
      nal handler  asynchronously and should not be assumed
      as reentrant.  If problems arise, move this  function
      out of any signal functions called by signal handlers
      to ensure POSIX compliance.
   */
   if (!realpath(path, rpath)) return errno;

   /* To assure  compatibility with POSIX  standards,  this
      function uses  open(),  read(),  lseek() and  close()
      instead of their file  stream  equivalents  (fopen(),
      fread(), fseek() and so on) because they are safe for
      use in asynchronous signal handling.
   */

   /* the return type of lseek() should always be  typecast
      because of differing return types from environment to
      environment. SVR1-3 returns long and BSD returns int.
      The configure script should set off_t to long if non-
      existant.
   */

   /* open the file */
   fd = open(rpath, O_RDONLY);
   if (fd < 0) return errno;

   /* read the file until eof */
   while (r) {
      r = read(fd, p, 1);
      /* skip to the beginning of a directive */
      while (r && (isspace(*p) || *p == '#' || comment)) {
         if (*p == '#') comment = 1;
         else if (*p == '\n') { comment = 0; line++; }
         r = read(fd, p, 1);
      }
      /* test for eof, if eof then no directive was found */
      if (!r) { close(fd); return 0; }

      /* get the current offset */
      curpos = (off_t)lseek(fd, 0, SEEK_CUR) - 1;
      if (curpos == -1) {
         close(fd);
         return errno;
      }

      /* find the length of the directive */
      for (n = 0; r && !isspace(*p); n++) r = read(fd, p, 1);

      /* test for eof, if eof then no directive was found */
      if (!n && !r) { close(fd); return 0; }

      /* go back to the beginning of the directive */
      err = (int)lseek(fd, curpos, SEEK_SET);
      if (err == curpos - 1) {
         close(fd);
         return errno;
      }

      /* allocate and read the directive into memory */
      if (n > dsize) {
         directive = (char *)realloc(directive, n + 1);
         if (!directive) {
            close(fd);
            return ENOMEM;
         }
         dsize = n;
      }
      r = read(fd, directive, n);

      /* terminate the string */
      directive[n] = 0;

      /* convert the directive to lower case (for
         case insensitive comparison) */
      strtolower(directive); /* in src/util/string_m.c */

      /* move to the value */
      while (r && isspace(*p) && *p != '\n') r = read(fd, p, 1);
      if (!r || *p == '\n' || *p == '#') {
         /* initialize the value if not done so already */
         if (!value) {
            value = (char *)malloc(1);
            if (!value) {
               close(fd);
               return ENOMEM;
            }
         }
         value[0] = 0;
         goto call_function;
      }

      /* get the current offset */
      curpos = (off_t)lseek(fd, 0, SEEK_CUR) - 1;
      if (curpos == -1) {
         close(fd);
         return errno;
      }

      /* find the end of the value */
      quote = 0; end = 0;
      while (r && !end) {

         /* check for keys where it could end */
         if (*p == '\"' || *p == '#' || *p == '\n') {

            /* get the current offset */
            charpos = (off_t)lseek(fd, 0, SEEK_CUR) - 1;
            if (charpos == -1) {
               close(fd);
               return errno;
            }

            /* go back so we can read the character before the key to check
               if the character is escaped (which means we should continue) */
            err = (int)lseek(fd, -2, SEEK_CUR);
            if (err == -3) {
               close(fd);
               return errno;
            }
            r = read(fd, p, 1);

            /* count how many back slashes there are (an odd number means
               that the key is escaped, even means otherwise) */
            for (n = 1; *p == '\\'; n++) {
               /* move back another offset to read */
               err = (int)lseek(fd, -2, SEEK_CUR);
               if (err == -3) {
                  close(fd);
                  return errno;
               }
               r = read(fd, p, 1);
            }

            /* move the offset back to the key and read it */
            err = (int)lseek(fd, charpos, SEEK_SET);
            if (err == charpos - 1) {
               close(fd);
               return errno;
            }
            r = read(fd, p, 1);

            /* if an even number of back slashes was counted meaning the key
               is not escaped, we should evaluate what to do */
            if (n & 1) {
               switch (*p) {
                  case '\"':
                     /* flag the current sequence of characters to follow as
                        being quoted (hashes are not considered comments) */
                     quote = !quote;
                     break;
                  case '#':
                     /* if we aren't in a quoted series, we've just hit an
                        in-line comment and have found the end of the value */
                     if (!quote) end = 1;
                     break;
                  case '\n':
                     /* new line characters must always be escaped, whether
                        inside a quoted series or not, otherwise they
                        terminate the value */
                     end = 1;
               }
            } else if (*p == '\n')
               /* escaped new line character. increment line counter */
               line++;
         }

         /* advance to the next character */
         r = read(fd, p, 1);
      }

      /* get the current offset */
      charpos = (off_t)lseek(fd, 0, SEEK_CUR) - 1;
      if (charpos == -1) {
         close(fd);
         return errno;
      }

      /* get the length of the value */
      n = charpos - curpos;
      if (r) n--; /* there is more to read, don't read the ending key */

      /* move offset back to the beginning of the value */
      err = (int)lseek(fd, curpos, SEEK_SET);
      if (err == curpos - 1) {
         close(fd);
         return errno;
      }

      /* allocate and read the value into memory */
      if (n > vsize) {
         value = (char *)realloc(value, n + 1);
         if (!value) {
            close(fd);
            return ENOMEM;
         }
         vsize = n;
      }
      r = read(fd, value, n);

      /* terminate the string */
      value[n] = 0;

      /* cut trailing whitespace off by termination */
      t = value + n;
      while (isspace(*--t)) *t = 0;

      /* escape the escaped quotes (replaceall is in src/util/string_m.c) */
      x = strcount(value, "\\\""); /* in src/util/string_m.c */
      if (x && n + x > vsize) {
         value = (char *)realloc(value, n + x + 1);
         if (!value) {
            close(fd);
            return ENOMEM;
         }
         vsize = n + x;
      }
      if (replaceall(value, "\\\"", "\\\\\"") < 0) {
         /* the replace operation failed for some unknown reason */
         close(fd); return errno;
      }

      /* remove all new line characters */
      if (replaceall(value, "\\\n", "") < 0) {
         /* the replace operation failed for some unknown reason */
         close(fd); return errno;
      }

      /* resolve escape sequences */
      strexpand(value); /* in src/util/string_m.c */

call_function:

      found = 0;

      /* loop through the array looking for a match for the value */
      for (n = 0; option[n].directive; n++) {
         err = fnmatch(option[n].directive, directive, FNM_NOESCAPE);
         if (!err) {
            /* found a match */
            found = 1;
            /* call the function for the array index item (if there is one) */
            if (option[n].action) {
               err = option[n].action(line, directive, value);
               if (err) { close(fd); return err; } /* return on error */
            }
         } else if (err != FNM_NOMATCH) {
            /* an error has occurred */
            close(fd);
            return err;
         }
      }
      if (!found && unknown) {
         /* no match was found for the value we read from the file
            call the function designated for unknown values (if defined) */
         err = unknown(line, directive, value);
         /* by returning on error, we give the function control over the
            operation of this function, allowing it to decided if we should
            continue reading the config file or close it and return */
         if (err) { close(fd); return err; } /* return on error */
      }

   }

   close(fd);
   return 0;
}
